SystemOrganization addCategory: #'Shout-Windows'!
SystemOrganization addCategory: #'Shout-Pluggable Views'!
SystemOrganization addCategory: #'Shout-Parsing'!
SystemOrganization addCategory: #'Shout-Styling'!
SystemOrganization addCategory: #'Shout-Text Support'!

!ChangeSorter methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:51'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"

	Preferences syntaxHighlightingAsYouType ifFalse: [^false].
	self showingSource ifFalse: [^false].
	self currentSelector ifNil: [^false].
	aTextStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !

PluggableTextController subclass: #PluggableShoutViewController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Pluggable Views'!

!PluggableShoutViewController methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 17:14'!
zapSelectionWith: aText 
	"Note edit except during typeIn, which notes edits at close."
	
	super zapSelectionWith: aText.
	beginTypeInBlock == nil
		ifFalse: [self userHasEdited]! !

!PluggableShoutViewController methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:34'!
deselectDuring: aZeroArgumentBlock
	"evaluate aZeroArgumentBlock with the receiver deselected.
	answer the receiver"
	| wasSelected |
	wasSelected := selectionShowing.
	self deselect.
	aZeroArgumentBlock value.
	wasSelected ifTrue:[self select]! !

!PluggableShoutViewController methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:33'!
clearParens
	lastParenLocation ifNotNil:
		[self text string size >= lastParenLocation ifTrue: [
			self text
				removeAttribute: TextEmphasis bold
				from: lastParenLocation
				to: lastParenLocation].
		lastParenLocation := nil]
! !

!PluggableShoutViewController methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 17:27'!
correctFrom: start to: stop with: aString
	
	view correctFrom: start to: stop with: aString.
	^super correctFrom: start to: stop with: aString! !

!PluggableShoutViewController methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:33'!
blinkParen
	lastParenLocation ifNotNil:
		[self text string size >= lastParenLocation ifTrue: [
			self text
				addAttribute: TextEmphasis bold
				from: lastParenLocation
				to: lastParenLocation]]! !

!Debugger methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:51'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"
	
	Preferences syntaxHighlightingAsYouType ifFalse: [^false].
	self showingSource ifFalse: [^false].
	aTextStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !

!Browser methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:50'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"
	| type |
	
	Preferences syntaxHighlightingAsYouType ifFalse: [^false].
	self showingSource ifFalse: [^false].
	type _ self editSelection.
	(#(newMessage editMessage editClass newClass) includes: type) ifFalse:[^false].
	aTextStyler classOrMetaClass: (type = #editClass ifFalse:[self selectedClassOrMetaClass]).
	^true! !

Object subclass: #SHTextStyler
	instanceVariableNames: 'sem backgroundProcess text monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Styling'!

!SHTextStyler commentStamp: 'tween 8/16/2004 16:04' prior: 0!
I am an Abstract class.
Subclasses of me can create formatted, coloured, and styled copies of Text that is given to them.
They may perform there styling in a background process which they create and manage.

My public protocol is...
	format: aText
	style: aText
	styleInBackgroundProcess: aText

I trigger the following events...
	#abouToStyle  - triggered when text is about to be styled (answering false will veto the styling)
	#styled: - triggered when styling has completed
	#styledInBackground: - triggered when styling in a background process has completed 
		
Subclasses of me should re-implement...

	rangesIn: - answer a sequence of SHRange describing the text tokens in a Text
	attributesFor: - answer a sequence of TextAttributes for a token type symbol  
	
Subclasses may optionally re-implement...
	basicFormat: aText
		The default is to just answer aText, but can be re-implemented to answer a modified copy of aText.
		This is performed once, before the first styling, but not on every keystroke in an editor.
		Its purpose is to allow the string of characters to be changed (i.e. string substitution), rather than simply styling the characters that are already there. So, for example, if you want the first word in some Text to be bold, you would do this in rangesIn:, but if you wanted the first word in some Text to be upperCase you would do this in basicFormat:
		(for example, SHTextStylerST80 changes all assigments to be leftArrow when a method is first displayed)	
	
	
!

!SHTextStyler methodsFor: 'styling' stamp: 'tween 5/9/2004 13:19'!
basicStyledTextFor: aText
	"Answer a copy of aText that is styled and formatted"
	| t |
	t := self basicFormat: aText.
	self basicStyle: t.
	^t! !

!SHTextStyler methodsFor: 'styling' stamp: 'tween 4/9/2004 13:03'!
style: aText
	self aboutToStyle
		ifTrue: [
			text := aText.
			self style]! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/28/2004 10:48'!
basicStyle: aText
	"style <aText>"
	| ranges |
	ranges := self rangesIn: aText .
	ranges ifNotNil: [self setAttributesIn: aText fromRanges: ranges]! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/24/2004 14:03'!
style
	self basicStyle: text.
	self triggerEvent: #styled: with: text! !

!SHTextStyler methodsFor: 'styling' stamp: 'tween 4/24/2004 14:02'!
styleInBackgroundProcess: aText
	self aboutToStyle
		ifTrue: [
			text _ aText.
			self styleInBackgroundProcess]
	! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 8/8/2004 11:16'!
veryDeepCopyWith: deepCopier
	"patch up the eventHandlers in the copy"
	|answer |
	answer := super veryDeepCopyWith: deepCopier.
	EventManager actionMaps at: answer put: (self actionMap veryDeepCopyWith: deepCopier).
	^answer ! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/9/2004 12:21'!
terminateBackgroundStylingProcess
	self monitor critical: [
		backgroundProcess 
			ifNotNil: [
				backgroundProcess terminate.
				backgroundProcess := nil].
		sem 
			ifNotNil:[
				sem terminateProcess.
				sem := nil].	
	]		! !

!SHTextStyler methodsFor: 'formatting' stamp: 'tween 4/9/2004 13:06'!
format: aText
	"Answer a copy of <aText> which has been reformatted,
	or <aText> if no formatting is to be applied"
	
	^self aboutToStyle
		ifTrue: [self basicFormat: aText]
		ifFalse: [aText]! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/28/2004 10:45'!
rangesIn: aText
	"Subclasses should re-implement this method to answer a Collection of SHRanges"

	self shouldBeImplemented! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/9/2004 12:13'!
monitor
	monitor isNil
		ifTrue: [monitor := Monitor new].
	^monitor! !

!SHTextStyler methodsFor: 'styling' stamp: 'tween 6/27/2004 12:23'!
shouldRemoveStylesFromAcceptText
	"Answer true if text should be unstyled before accepting, false otherwise.
	Triggering the #aboutToStyle event is the simplest way of determining, this
	since if the text is not styled on changes it does not need to be unstyled, and
	vice-versa"

	^self triggerEvent: #aboutToStyle ifNotHandled: [false]
	

	! !

SHTextStyler subclass: #SHTextStylerST80
	instanceVariableNames: 'classOrMetaClass workspace font parser formatAssignments environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Styling'!

SHTextStylerST80 class
	instanceVariableNames: 'styleTable textAttributesByPixelHeight'!

!SHTextStylerST80 commentStamp: 'tween 8/16/2004 16:29' prior: 0!
I style Smalltalk methods and expressions.

My 'styleTable' class instance var holds an array ofArrays which control how each token is styled/coloured. See my defaultStyleTable class method for its structure.
My styleTable can be changed by either modifying the defaultStyleTable class method and then executing SHTextStylerST80 initialize ; or by giving me a new styleTable with my styleTable: class method.

My 'textAttributesByPixelSize' class instance var contains a dictionary of dictionaries.
	The key is a pixelSize and the value a Dictionary from token type Symbol to TextAttribute array.
	It is created/maintained automatically.
	
I also install these 3 preferences when my class initialize method is executed....
	#syntaxHighlightingAsYouType  - controls whether methods are styled in browsers
	#syntaxHighlightingAsYouTypeAnsiAssignment - controls whether assignments are formatted to be :=
	#syntaxHighlightingAsYouTypeLeftArrowAssignment - controls whether assignments are formatted to be _
	
I can unstyle method source (so that styling information is not stored when a method is saved), using my #unstyledTextFrom: method. (TextActions are preserved when a method is unstyled).


	
	
	
	
	 
	
!

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 8/16/2004 16:20'!
styleTable: anArray
	"Set the receiver's styleTable to anArray.
	Clear textAttributesByPixelSize cache so that it will reinitialize.	 
	" 
	
	styleTable := nil.
	textAttributesByPixelHeight := nil! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/25/2004 18:27'!
applyTextActionsInRunArray: aRunArray to: aText
	"Examine each run in aRunArray, and if that run includes a TextAction then add ALL the attributes for that run to aText for the same range of characters. This preserves both the TextAction and any associated styling such as color and emphasis"
	aRunArray withStartStopAndValueDo:[:start :stop :attribs|
		(attribs detect: [:each | each isKindOf: TextAction] ifNone:[nil])
			ifNotNil:[
				attribs do: [:eachAttrib | aText addAttribute: eachAttrib from: start to: stop]]]. ! !

!SHTextStylerST80 methodsFor: 'converting' stamp: 'tween 4/24/2004 15:25'!
unstyledTextFrom: aText
	| answer |	
	answer _ Text fromString: aText string.
	"restore attributes for TextActions so they can still be seen and clicked on"
	self applyTextActionsInRunArray: aText runs to: answer.
	^answer! !

!SHTextStylerST80 methodsFor: 'accessing' stamp: 'tween 5/9/2004 12:50'!
environment: anObject
	environment := anObject! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/24/2004 14:04'!
attributesFor: aSymbol 
	^self class attributesFor: aSymbol pixelHeight: self pixelHeight
	! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/28/2004 10:45'!
convertAssignmentsToLeftArrow: aText
	"If the Preference is to show leftArrowAssignments then answer a copy of  <aText> where each ansi assignment (:=) is replaced with a left arrow. A parser is used so that each ':=' is only replaced if it actually occurs within an assigment statement"

	^self replaceStringForRangesWithType: #ansiAssignment with: '_' in: aText! !

!SHTextStylerST80 class methodsFor: 'instance creation' stamp: 'tween 5/1/2004 14:31'!
new
	^super new
		initialize;
		yourself! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/28/2004 14:22'!
setAttributesIn: aText fromRanges: ranges
	|  oldRuns | 		
	oldRuns := aText runs.
	super setAttributesIn: aText fromRanges: ranges.
	"restore attributes for TextActions so they can still be seen and clicked on"
	self applyTextActionsInRunArray: oldRuns to: aText
	! !

!SHTextStylerST80 class methodsFor: 'preferences' stamp: 'tween 6/27/2004 11:33'!
initializePreferences
	
	(Preferences preferenceAt: #syntaxHighlightingAsYouType) ifNil:[
		Preferences 
			disable: #colorWhenPrettyPrinting;
			disable: #browseWithPrettyPrint.
		Preferences 
			addPreference: #syntaxHighlightingAsYouType
			 categories: #( browsing)
			default: true 
			balloonHelp: 'Enable, or disable, Shout - Syntax Highlighting As You Type. When enabled, code in Browsers and Workspaces is styled to reveal its syntactic structure. When the code is changed (by typing some characters, for example), the styling is changed so that it remains in sync with the modified code'
			projectLocal: false 
			changeInformee: nil 
			changeSelector: nil.].
	(Preferences preferenceAt: #syntaxHighlightingAsYouTypeAnsiAssignment) ifNil:[
		Preferences 
			addPreference: #syntaxHighlightingAsYouTypeAnsiAssignment
			 categories: #( browsing)
			default: (Preferences ansiAssignmentOperatorWhenPrettyPrinting) 
			balloonHelp: 'If true, and syntaxHighlightingAsYouType is enabled,  all left arrow assignments ( _ ) will be converted to the ANSI format ( := ) when a method is selected in a Browser. Whilst editing a method, this setting has no effect - both the left arrow and the ansi format may be used'
			projectLocal: false 
			changeInformee: self
			changeSelector: #ansiAssignmentPreferenceChanged].		
	(Preferences preferenceAt: #syntaxHighlightingAsYouTypeLeftArrowAssignment) ifNil:[
		Preferences 
			addPreference: #syntaxHighlightingAsYouTypeLeftArrowAssignment
		 	categories: #( browsing)
			default: (Preferences ansiAssignmentOperatorWhenPrettyPrinting not) 
			balloonHelp: 'If true, and syntaxHighlightingAsYouType is enabled,  all ANSI format assignments ( := ) will be converted to left arrows ( _ ) when a method is selected in a Browser. Whilst editing a method, this setting has no effect - both the left arrow and the ansi format may be used'
			projectLocal: false 
			changeInformee: self 
			changeSelector: #leftArrowAssignmentPreferenceChanged ].							! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 4/18/2004 21:10'!
attributesFor: aSymbol pixelHeight: aNumber
	
	(self textAttributesByPixelHeight includesKey: aNumber)
		ifFalse:[self initializeTextAttributesForPixelHeight: aNumber].
	^(self textAttributesByPixelHeight at: aNumber) at: aSymbol ifAbsent:[nil]! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/28/2004 10:42'!
rangesIn: aText
	^self rangesIn: aText setWorkspace: true

	! !

!SHTextStylerST80 methodsFor: 'accessing' stamp: 'tween 4/9/2004 12:47'!
classOrMetaClass: aBehavior
	classOrMetaClass := aBehavior! !

!SHTextStylerST80 methodsFor: 'accessing' stamp: 'tween 4/18/2004 10:32'!
font: aFont
	font := aFont! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 4/25/2004 12:28'!
initializeTextAttributesForPixelHeight: aNumber
	| d element color  textStyleName  pixelHeight emphasis font textStyle attrArray |
	 
	self textAttributesByPixelHeight at: aNumber put: (d := IdentityDictionary new).
	self styleTable do: [:each |
		element := each first.
		color := each at: 2 ifAbsent:[nil].
		color:=color ifNotNil: [Color colorFrom: color].
		emphasis := each at: 3 ifAbsent:[nil].
		emphasis ifNil: [emphasis := #normal].
		textStyleName := each at: 4 ifAbsent: [nil].
		pixelHeight := each at: 5 ifAbsent: [aNumber].	
		textStyleName ifNil:[pixelHeight := nil].	
		textStyle := TextStyle named: textStyleName.
		font := textStyle ifNotNil:[pixelHeight ifNotNil:[textStyle fontOfSize: pixelHeight]].
		attrArray := self attributeArrayForColor: color emphasis: emphasis font: font.
		attrArray notEmpty 
			ifTrue:[
				d at: element put: attrArray]]	
	! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 4/18/2004 20:44'!
attributeArrayForColor: aColorOrNil emphasis: anEmphasisSymbolOrArrayorNil font: aTextStyleOrFontOrNil
	"Answer a new Array containing any non nil TextAttributes specified"
	| answer emphArray |

	answer := Array new.
	aColorOrNil ifNotNil: [answer := answer, {TextColor color: aColorOrNil}].
	anEmphasisSymbolOrArrayorNil ifNotNil: [
		emphArray := anEmphasisSymbolOrArrayorNil isSymbol 
			ifTrue: [{anEmphasisSymbolOrArrayorNil}] 
			ifFalse: [anEmphasisSymbolOrArrayorNil].
		emphArray do: [:each |
			answer := answer, {TextEmphasis perform: each}]].
	aTextStyleOrFontOrNil ifNotNil: [
		answer := answer, {TextFontReference toFont: aTextStyleOrFontOrNil}].
	^answer! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/25/2004 11:56'!
pixelHeight
	"In Morphic the receiver will have been given a code font, in MVC the font will be nil. So when the font is nil, answer the pixelHeight of the MVC Browsers' code font, i.e. TextStyle defaultFont pixelHeight"
	^(font ifNil:[TextStyle defaultFont]) pixelSize! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 8/24/2004 16:06'!
basicFormat: aText
	"Perform any formatting of aText necessary and answer either aText, or a formatted copy of aText"
	| t |
	t := aText.
	aText asString = Object sourceCodeTemplate
		ifTrue:[
			"the original source code template does not parse, replace it with one that does"
			t := self sourceCodeTemplate].
	formatAssignments
		ifTrue:[
			Preferences syntaxHighlightingAsYouTypeAnsiAssignment 
				ifTrue:[^self convertAssignmentsToAnsi: t].
			Preferences syntaxHighlightingAsYouTypeLeftArrowAssignment 
				ifTrue:[^self convertAssignmentsToLeftArrow: t]].		
	^t! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/28/2004 14:22'!
replaceStringForRangesWithType: aSymbol with: aString in: aText 
	"Answer aText if no replacements, or a copy of aText with each range with a type of aSymbol replaced by aString"
	| answer toReplace indexShift start end |
	toReplace := (self rangesIn: aText setWorkspace: false) 
		select: [:each | each type = aSymbol].
	toReplace isEmpty ifTrue: [^aText].
	answer := aText copy.
	indexShift := 0.
	(toReplace asSortedCollection: [:a :b | a start <= b start]) 
		do: [:each | 
			start := each start + indexShift.
			end := each end + indexShift.
			answer 	replaceFrom: start to: end with: aString.
			indexShift := indexShift + (aString size - each length)].
	^answer! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 4/18/2004 21:07'!
styleTable
	
	styleTable ifNotNil:[^styleTable].
	^styleTable := self defaultStyleTable.	
						! !

!SHTextStylerST80 class methodsFor: 'class initialization' stamp: 'tween 4/27/2004 20:06'!
initialize  
	"Clear styleTable and textAttributesByPixelSize cache so that they will 
	reinitialize.	 

		SHTextStylerST80 initialize
	" 
	
	styleTable := nil.
	textAttributesByPixelHeight := nil.	
	self initializePreferences! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 5/9/2004 12:50'!
rangesIn: aText setWorkspace: aBoolean
	"Answer a collection of SHRanges by parsing aText.
	When formatting it is not necessary to set the workspace, and this can make the parse take less time, so aBoolean specifies whether the parser should be given the workspace"
	parser ifNil: [parser := SHParserST80 new].
	parser workspace: (aBoolean ifTrue:[workspace]).
	environment ifNotNil: [parser environment: environment].
	^parser
		classOrMetaClass: classOrMetaClass;
		source: aText asString;
		parse;
		ranges
! !

!SHTextStylerST80 methodsFor: 'accessing' stamp: 'tween 5/1/2004 14:33'!
formatAssignments: aBoolean
	"determines whether assignments are reformatted according to the Preferences,
	or left as they are"
	formatAssignments := aBoolean! !

!SHTextStylerST80 methodsFor: 'initialize-release' stamp: 'tween 5/1/2004 14:31'!
initialize
	formatAssignments := true! !

!SHTextStylerST80 class methodsFor: 'preferences' stamp: 'tween 6/27/2004 11:31'!
leftArrowAssignmentPreferenceChanged
	"the user has changed the syntaxHighlightingAsYouTypeLeftArrowAssignment setting.
	If they have turned it on then force syntaxHighlightingAsYouTypeAnsiAssignment
	to be turned off"
	Preferences syntaxHighlightingAsYouTypeLeftArrowAssignment 
		ifTrue:[Preferences disable: #syntaxHighlightingAsYouTypeAnsiAssignment]! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 4/28/2004 10:44'!
convertAssignmentsToAnsi: aText
	"If the Preference is to show ansiAssignments then answer a copy of  <aText> where each  left arrow assignment is replaced with a ':=' ansi assignment. A parser is used so that each left arrow is only replaced if it occurs within an assigment statement"

	^self replaceStringForRangesWithType: #assignment with: ':=' in: aText! !

!SHTextStylerST80 methodsFor: 'private' stamp: 'tween 8/24/2004 16:05'!
sourceCodeTemplate
	"Answer an expression to be edited and evaluated in order to define 
	methods in this class."

	^'messageSelectorAndArgumentNames
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 4/18/2004 21:10'!
textAttributesByPixelHeight
	
	textAttributesByPixelHeight ifNotNil:[^textAttributesByPixelHeight].
	^textAttributesByPixelHeight := Dictionary new
						! !

!SHTextStylerST80 class methodsFor: 'style table' stamp: 'tween 5/7/2004 08:00'!
defaultStyleTable
	"color can be a valid argument to Color class>>colorFrom: , or nil to
	use the editor text color.
	Multiple emphases can be specified using an array e.g. #(bold italic).
	If emphasis is not specified, #normal will be used.
	if pixel height is not specified , then the editor font size will be used.
	"
								
^#(
	"(symbol	color		[emphasisSymbolOrArray		[textStyleName [pixelHeight]]])"		
	(default 								black 		)
	(invalid 								red 			)
	(excessCode 								red 			)
	(comment 								(green muchDarker) 		italic)
	(unfinishedComment 					(red muchDarker) 	italic)
	(#'$'									(red muchDarker) 	)
	(character								(red muchDarker) 	)
	(integer									(red muchDarker) 	)
	(number								(red muchDarker) 	)	
	(#-										(red muchDarker) 	)
	(symbol									(blue muchDarker)			bold)	
	(stringSymbol							(blue muchDarker)			bold)	
	(literalArray							(blue muchDarker)			bold)
	(string									(magenta muchDarker)	normal			'Accuny')
	(unfinishedString						red			normal			'Accuny')
	(assignment								nil			bold			'Accuny')
	(ansiAssignment 						nil			bold)
	(literal									nil			italic)
	(keyword								(blue muchDarker)			)
	(binary 								(blue muchDarker)			)	
	(unary									(blue muchDarker)			)
	(incompleteKeyword						(gray muchDarker)			underlined)
	(incompleteBinary 						(gray muchDarker)			underlined)	
	(incompleteUnary						(gray muchDarker)			underlined)
	(undefinedKeyword						red			)
	(undefinedBinary 						red			)	
	(undefinedUnary						red			)													
	(patternKeyword 						nil			bold)
	(patternBinary 							nil			bold)
	(patternUnary							nil			bold)	
	(#self 									(red muchDarker)	bold)
	(#super									(red muchDarker)	bold) 
	(#true 									(red muchDarker)	bold)
	(#false 									(red muchDarker)	bold)
	(#nil 									(red muchDarker)	bold)
	(#thisContext 							(red muchDarker)	bold)
	(#return								(red muchDarker)	bold)
	(patternArg 							(blue muchDarker)			italic)	
	(methodArg 								(blue muchDarker)			italic)
	(blockPatternArg 						(blue muchDarker)			italic)
	(blockArg 								(blue muchDarker)			italic)
	(argument 								(blue muchDarker)			italic)
	(blockArgColon							black		) 
	(leftParenthesis							black		) 
	(rightParenthesis						black		) 
	(leftParenthesis1						(green muchDarker)		) 
	(rightParenthesis1						(green muchDarker)		) 
	(leftParenthesis2						(magenta muchDarker)		) 
	(rightParenthesis2						(magenta muchDarker)		) 
	(leftParenthesis3						(red muchDarker)		) 
	(rightParenthesis3						(red muchDarker)		) 
	(leftParenthesis4						(green darker)		) 
	(rightParenthesis4						(green darker)		) 
	(leftParenthesis5						(orange darker)		) 
	(rightParenthesis5						(orange darker)		) 
	(leftParenthesis6						(magenta darker)		) 
	(rightParenthesis6						(magenta darker)		) 
	(leftParenthesis7						blue		) 
	(rightParenthesis7						blue		) 
	(blockStart 								black		) 
	(blockEnd 								black		) 
	(blockStart1								(green muchDarker)			) 
	(blockEnd1								(green muchDarker)			) 
	(blockStart2								(magenta muchDarker)		) 
	(blockEnd2								(magenta muchDarker)		) 
	(blockStart3								(red muchDarker)		) 
	(blockEnd3								(red muchDarker)		) 
	(blockStart4								(green darker)		) 
	(blockEnd4								(green darker)		) 
	(blockStart5								(orange darker)		) 
	(blockEnd5								(orange darker)		) 
	(blockStart6								(magenta darker)		) 
	(blockEnd6								(magenta darker)		) 
	(blockStart7								blue		) 
	(blockEnd7								blue		) 																																																		
	(arrayStart								black		) 
	(arrayEnd								black		) 
	(arrayStart1							black		) 
	(arrayEnd1								black		) 
	(leftBrace 								black		) 
	(rightBrace 								black		) 
	(cascadeSeparator 						black		) 
	(statementSeparator 						black		) 
	(externalCallType 						black		) 
	(externalCallTypePointerIndicator 		black		) 
	(primitiveOrExternalCallStart 			black		) 
	(primitiveOrExternalCallEnd				black		)
	(methodTempBar							gray		) 
	(blockTempBar 							gray		)
	(blockArgsBar							gray		)
	(primitive								(green muchDarker)		bold)
	(externalFunctionCallingConvention		(green muchDarker)		bold) 
	(module									(green muchDarker)		bold)
	(blockTempVar 							gray		italic)
	(blockPatternTempVar					gray		italic)
	(instVar 								black		bold)
	(workspaceVar							black		bold)
	(undefinedIdentifier						red			bold)
	(incompleteIdentifier					(gray darker) 	(italic underlined))
	(tempVar								(gray darker)	italic)
	(patternTempVar						(gray darker)	italic)
	(poolConstant							(gray darker)	italic)
	(classVar								(gray darker)	bold)
	(globalVar								black		bold) )
							! !

!SHTextStylerST80 class methodsFor: 'preferences' stamp: 'tween 6/27/2004 11:30'!
ansiAssignmentPreferenceChanged
	"the user has changed the syntaxHighlightingAsYouTypeAnsiAssignment setting.
	If they have turned it on then force syntaxHighlightingAsYouTypeLeftArrowAssignment
	to be turned off"
	Preferences syntaxHighlightingAsYouTypeAnsiAssignment 
		ifTrue:[Preferences disable: #syntaxHighlightingAsYouTypeLeftArrowAssignment]! !

!SHTextStylerST80 methodsFor: 'accessing' stamp: 'tween 4/15/2004 13:23'!
workspace: aWorkspace
	workspace := aWorkspace! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/24/2004 14:04'!
aboutToStyle
	self terminateBackgroundStylingProcess.
	^self triggerEvent: #aboutToStyle ifNotHandled: [false]
	

	! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/28/2004 13:48'!
setAttributesIn: aText fromRanges: ranges
	| charAttr defaultAttr attr newRuns newValues lastAttr | 		

	defaultAttr := self attributesFor: #default.
	charAttr := Array new: aText size.
	1 to: charAttr size do: [:i | charAttr at: i put: defaultAttr].
	ranges do: [:range |
		(attr := self attributesFor: range type) ifNotNil:[
			range start to: range end do: [:i | charAttr at: i put: attr]]].
	newRuns := OrderedCollection new: charAttr size // 10.
	newValues := OrderedCollection new: charAttr size // 10.
	1 to: charAttr size do: [:i |
		i = 1 
			ifTrue: [
				newRuns add: 1.
				lastAttr := newValues add: (charAttr at: i)]
			ifFalse:[
				(charAttr at: i) = lastAttr
					ifTrue: [newRuns at: newRuns size put: (newRuns last + 1)]
					ifFalse: [
						newRuns add: 1.
						lastAttr := newValues add: (charAttr at: i)]]].	
	aText runs: (RunArray runs: newRuns values: newValues)
	! !

!SHTextStyler methodsFor: 'styling' stamp: 'tween 5/9/2004 13:21'!
styledTextFor: aText
	"Answer a copy of aText that is styled and formatted,
	or aText if #aboutToStyle event aborted styling"
	
	^self aboutToStyle
		ifTrue: [self basicStyledTextFor: aText]
		ifFalse:[aText]
! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/24/2004 14:04'!
basicFormat: aText
	"Answer a formatted copy of <aText>, or <aText> if no formatting is applied"
	"default behaviour is to answer <aText>"
	^aText! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/24/2004 19:32'!
styleInBackgroundProcess
	self monitor critical: [
		sem := Semaphore new. 
		[sem notNil
			ifTrue: [
				sem wait.
				self triggerEvent: #styledInBackground: with: text]
		] forkAt: Processor activePriority.
		backgroundProcess := 
			[self basicStyle: text.
			sem signal]
				forkAt: Processor userBackgroundPriority.
	]! !

!SHTextStyler methodsFor: 'private' stamp: 'tween 4/28/2004 10:50'!
attributesFor: aSymbol 
	"answer a collection of TextAttributes used to style ranges whose type is <aSymbol>"
	self shouldBeImplemented

	! !

Object subclass: #SHParserST80
	instanceVariableNames: 'classOrMetaClass source workspace arguments sourcePosition currentToken currentTokenFirst temporaries instanceVariables errorBlock currentTokenSourcePosition blockDepth bracketDepth ranges environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Parsing'!

!SHParserST80 commentStamp: 'tween 8/16/2004 15:44' prior: 0!
I am a Smalltalk method / expression parser.

Rather than creating an Abstract Syntax Tree, I create a sequence of SHRanges (in my 'ranges' instance variable), which represent the tokens within the String I am parsing.

I am used by a SHTextStylerST80 to parse method source strings.
I am able to parse incomplete / incorrect methods, and so can be used to parse methods that are being edited.

My 'source' instance variable should be set to the string to be parsed.

My 'classOrMetaClass' instance var must be set to the class or metaClass for the method source so that I can correctly resolve identifiers within the source. If this is nil , I parse the source as an expression (i.e. a doIt expression).

My 'workspace' instance variable can be set to a Workspace, so that I can resolve workspace variables.

My 'environment' instance variable is the global namespace (this is initialized to Smalltalk, but can be set to a different environment).

Example 1.
	ranges := SHParserST80 new
		classOrMetaClass: Object;
		source: 'testMethod ^self';
		parse;
		ranges
		
!

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 09:44'!
parseStatement
	currentTokenFirst = $^ ifTrue: [self scanPast: #return].
	self parseExpression! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 09:56'!
scanPast: rangeType start: startInteger end: endInteger
	"record rangeType for current token from startInteger to endInteger,
	 and scanNext token"

	^self 
		rangeType: rangeType start: startInteger end: endInteger;
		scanNext
	
! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 4/27/2004 18:59'!
source: aString
    source := aString! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 15:00'!
scanNumber
	| start c nc |
	start := sourcePosition.
	self skipDigits.
	c := self currentChar.
	c = $r 
		ifTrue: [
			self skipBigDigits.
			currentToken := source copyFrom: start to: sourcePosition - 1.
			^currentTokenSourcePosition := start].
	c = $s 
		ifTrue: [
			self skipDigits.
			currentToken := source copyFrom: start to: sourcePosition - 1.
			^currentTokenSourcePosition := start].
	c = $. 
		ifTrue: [
			self nextChar isDigit 
				ifFalse: [
					sourcePosition := sourcePosition - 1.
					currentToken := source copyFrom: start to: sourcePosition - 1.
					^currentTokenSourcePosition := start]
				ifTrue: [self skipDigits.].
			c := self currentChar.
			(#($d $e $q) includes: c) 
				ifTrue: [
					((nc := self nextChar) isDigit or: [nc = $-]) 
						ifTrue: [
							self skipDigits.
							currentToken := source copyFrom: start to: sourcePosition - 1.
							^currentTokenSourcePosition := start]].
			c = $s ifTrue: [self skipDigits]].
	currentToken := source copyFrom: start to: sourcePosition - 1.
	^currentTokenSourcePosition := start! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 13:51'!
parseExpression
	| assignType |
	self isName 
		ifTrue: [
			self scanPast: (self resolve: currentToken).
			self isAssignment 
				ifTrue: [
					assignType := self isAnsiAssignment 
						ifTrue: [#ansiAssignment]
						ifFalse: [#assignment].
					self scanPast: assignType.
					self parseExpression]
				ifFalse: [self parseCascade]]
		ifFalse: [
			self parseTerm.
			self parseCascade]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 5/7/2004 09:33'!
parseUnary
	| unary type |
	
    [self isName]
        whileTrue: [
			unary := currentToken.
			type := #unary.
			(unary isEmpty or:[Symbol hasInterned: unary ifTrue: [:sym | ]])
				ifFalse:[
					type := (Symbol thatStartsCaseSensitive: unary skipping: nil) isNil
						ifTrue: [#undefinedUnary]
						ifFalse:[#incompleteUnary]].
			self scanPast: type]
! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 13:59'!
parsePrimitiveOrExternalCall
	currentTokenFirst = $< ifFalse: [^nil].
	self scanPast: #primitiveOrExternalCallStart.
	currentToken = 'primitive:' 
		ifTrue: [
			self rangeType: #primitive.
			^self parsePrimitive].
	self isTokenExternalFunctionCallingConvention 
		ifTrue: [
			self rangeType: #externalFunctionCallingConvention.
			^self parseExternalCall].
	self error	": 'Invalid External Function Calling convention'"! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 13:58'!
parsePrimitive
	self scanNext.
	currentTokenFirst isDigit 
		ifTrue: [self scanPast: #integer]
		ifFalse: [
			self failUnless: [currentTokenFirst = $'].
			self parseString.
			currentToken = 'module:' 
				ifTrue: [
					self scanPast: #module.
					self failUnless: [currentTokenFirst = $'].
					self parseString]].
	self failUnless: [currentToken = '>'].
	self scanPast: #primitiveOrExternalCallEnd! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 4/15/2004 13:18'!
workspace: aWorkspace
    workspace := aWorkspace! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 09:42'!
peekChar
	^source at: sourcePosition + 1 ifAbsent: [$ ]! !

!SHParserST80 methodsFor: 'character testing' stamp: 'tween 4/17/2004 11:16'!
isBigDigit: aCharacter
    "Answer true if aCharacter is a digit or a capital
    letter"

    ^aCharacter digitValue >= 0! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 09:44'!
isKeyword
	^currentTokenFirst isLetter and: [currentToken last = $:]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 13:57'!
parseLiteralArrayElement
	currentTokenFirst isLetter 
		ifTrue: [
			| type |
			type := (#('true' 'false' 'nil') includes: currentToken) 
				ifTrue: [currentToken asSymbol]
				ifFalse: [#symbol].
			^self scanPast: type].
	currentTokenFirst = $( 
		ifTrue: [
			self scanPast: #arrayStart.
			^self parseArray].
	^self parseLiteral: true! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 5/9/2004 12:43'!
environment: anObject
	environment := anObject! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:25'!
isMethodTempName: aString 
	"Answer true if aString is the name of a method temporary, false otherwise.
    Does not check whether aString is also a block temporary
    or argument"

	((arguments at: 0 ifAbsent: [#()]) includes: aString) ifTrue: [^false].
	^(temporaries at: 0 ifAbsent: [#()]) includes: aString! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:58'!
skipBigDigits
	[self isBigDigit: self nextChar] 
		whileTrue: []
! !

!SHParserST80 methodsFor: 'character testing' stamp: 'tween 4/19/2004 18:17'!
isSelectorCharacter: aCharacter

	aCharacter isAlphaNumeric ifTrue: [^false].
	aCharacter isSeparator ifTrue:[^false].
	"$- is specified here as NOT being a selector char, but it can appear as the 
	first char in a binary selector. That case is handled specially elsewhere"
	(#( $" $# $$ $' $: $( $) $. $; $[ $] ${ $} $^ $_ $- ) includes: aCharacter) 
		ifTrue:[^false].
	aCharacter asciiValue = 30 ifTrue: [^false "the doIt char"].
	aCharacter asciiValue = 0 ifTrue: [^false].
	"Any other char is ok as a binary selector char."
	^true
! !

!SHParserST80 methodsFor: 'recording ranges' stamp: 'tween 4/28/2004 10:20'!
rangeType: aSymbol start: s end: e 
	^ranges add: (SHRange start: s end: e type: aSymbol)! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 09:43'!
isAssignment
	^currentToken = ':=' or: [currentToken = '_']! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/17/2004 22:17'!
parseUnaryMessagePattern
	
	 self scanPast: #patternUnary
! !

!SHParserST80 class methodsFor: 'instance creation' stamp: 'tween 5/9/2004 12:44'!
new
	^super new
		initialize;
		yourself! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:13'!
scanComment
	| c s e |
	s := sourcePosition.
	
	[sourcePosition := sourcePosition + 1.
	(c := self currentChar) 
		ifNil: [
			self rangeType: #unfinishedComment start: s end: source size.
			^self error	": 'unfinished comment'"].
	c = $"] 
		whileFalse: [].
	e := sourcePosition.
	s < e ifTrue: [self rangeType: #comment start: s end: e].
	self nextChar.
	self scanWhitespace! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 7/5/2004 14:37'!
scanBinary
	| c d |
	c := self currentChar.
	currentTokenSourcePosition := sourcePosition.
	currentToken := c asString.
	d := self nextChar.
	((self isSelectorCharacter: c) or: [c = $: or: [c = $-]]) ifFalse: [^currentToken].
	(c = $: and: [d = $=]) 
		ifTrue: [" := assignment"
			currentToken := currentToken , d asString.
			self nextChar.
			^currentToken].
	c = $| ifTrue:["| cannot precede a longer token" ^currentToken].
	[self isSelectorCharacter: d] 
		whileTrue: [
			currentToken := currentToken , d asString.
			d := self nextChar].
	^currentToken! !

!SHParserST80 methodsFor: 'error handling' stamp: 'tween 4/28/2004 10:16'!
error
	self 
		rangeType: #excessCode
		start: (ranges isEmpty ifTrue: [1] ifFalse: [ranges last end + 1])
		end: source size.
	errorBlock value! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/17/2004 11:29'!
parse
    "Parse the receiver's text as a Smalltalk method"

    ^self parse: (classOrMetaClass notNil) ! !

!SHParserST80 methodsFor: 'parse support' stamp: 'tween 4/28/2004 10:08'!
initializeInstanceVariables
	instanceVariables := classOrMetaClass notNil 
		ifTrue: [classOrMetaClass allInstVarNames asArray]
		ifFalse: [Set new]! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 4/28/2004 10:17'!
ranges
	^ranges! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:14'!
scanNext
	self scanWhitespace.
	currentTokenFirst := self currentChar.
	currentTokenFirst isNil 
		ifTrue: [" end of input "
			currentTokenFirst := $ .
			currentTokenSourcePosition := nil.
			currentToken := nil.
			^nil].
	currentTokenFirst isDigit ifTrue: [^self scanNumber].
	currentTokenFirst isLetter ifTrue: [^self scanIdentifier].
	^self scanBinary! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/18/2004 12:24'!
parseMessagePattern   

	self isName 
		ifTrue: [self parseUnaryMessagePattern]
		ifFalse: [
			self isBinary
				ifTrue:[self parseBinaryMessagePattern]
				ifFalse:[
					self failUnless:[self isKeyword].
					self parseKeywordMessagePattern]]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/18/2004 12:24'!
parseKeywordMessagePattern   

	[self isKeyword]
		whileTrue: [ 
			self scanPast:  #patternKeyword. 
			self failUnless:[self isName].
			self scanPast: #patternArg]

! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 09:43'!
isTokenExternalFunctionCallingConvention
	| descriptorClass |
	descriptorClass := Smalltalk at: #ExternalFunction ifAbsent: [nil].
	descriptorClass == nil ifTrue: [^false].
	^(descriptorClass callingConventionFor: currentToken) notNil! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:27'!
parseCascade
	self parseKeyword.
	[currentTokenFirst = $;] 
		whileTrue: [
			self scanPast: #cascadeSeparator.
			self parseKeyword]! !

!SHParserST80 methodsFor: 'recording ranges' stamp: 'tween 4/28/2004 09:54'!
rangeType: aSymbol 
	^self 
		rangeType: aSymbol
		start: currentTokenSourcePosition
		end: currentTokenSourcePosition + currentToken size - 1! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 4/17/2004 22:21'!
source
	^source! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 13:52'!
parseExternalCall
	self scanNext.
	self scanPast: #externalCallType.
	currentToken = '*' 
		ifTrue: [self scanPast: #externalCallTypePointerIndicator].
	currentTokenFirst isDigit 
		ifTrue: [self scanPast: #integer]
		ifFalse: 	[
			self failUnless: [currentTokenFirst = $'].
			self parseString].
	self failUnless: [currentTokenFirst = $(].
	self scanPast: #leftParenthesis.
	[currentTokenFirst ~= $)] 
		whileTrue: [
			self scanPast: #externalCallType.
			currentToken = '*' 
				ifTrue: [self scanPast: #externalCallTypePointerIndicator]].
	self scanPast: #rightParenthesis.
	currentToken = 'module:' 
		ifTrue: [
			self scanPast: #module.
			self failUnless: [currentTokenFirst = $'].
			self parseString].
	self failUnless: [currentToken = '>'].
	self scanPast: #primitiveOrExternalCallEnd! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:26'!
isMethodArgName: aString 
	"Answer true if aString is the name of a method argument, false otherwise.
    Does not check whether aString is also a blockArgName"

	^(arguments at: 0 ifAbsent: [#()]) includes: aString! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 09:48'!
parseBlock
	self enterBlock.
	self scanPast: #blockStart level: bracketDepth.
	currentTokenFirst = $: ifTrue: [self parseBlockArguments].
	currentTokenFirst = $| ifTrue: [self parseBlockTemporaries].
	self parseStatementList.
	self failUnless: [currentTokenFirst = $]].
	self scanPast: #blockEnd level: bracketDepth.
	self leaveBlock! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:06'!
parseSymbolIdentifier
	| c start end |
	c := self currentChar.
	self failUnless: [c isLetter or: [c = $:]].
	start := sourcePosition.	
	[c := self nextChar.
	c isAlphaNumeric or: [c = $:]] 
		whileTrue: [].
	end := sourcePosition - 1.
	c := source copyFrom: start - 1 to: end.
	self scanPast: #symbol start: start - 1 end: end.
	^c! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:37'!
isIncompleteBlockArgName: aString 
	"Answer true if aString is the start of the name of a block argument, false otherwise"
	|  arg |
	blockDepth to: 1 by: -1 do: [:level | 
		arg := (arguments at: level ifAbsent: [#()]) anySatisfy: [:each | each beginsWith: aString].
		arg ifTrue: [^true]].
	^false! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:06'!
parseSymbolSelector
	| start end |
	start := sourcePosition - 1.
	end := sourcePosition.
	[self isSelectorCharacter: self nextChar] 
		whileTrue: [end := sourcePosition].
	self scanPast: #symbol start: start end: end! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:24'!
isBlockArgName: aString 
	"Answer true if aString is the name of a block argument, false otherwise"
	| temp arg |
	blockDepth to: 1 by: -1 do: [:level | 
		arg := (arguments at: level ifAbsent: [#()]) includes: aString.
		arg ifTrue: [^true].
		temp := (temporaries at: level ifAbsent: [#()]) includes: aString.
		temp ifTrue: [^false]].
	^false! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:57'!
skipDigits
	[self nextChar isDigit] 
		whileTrue: []! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:09'!
parseSymbolString
	| first c last |
	first := sourcePosition.
	self nextChar.
	[(c := self currentChar) isNil 
		ifTrue: [
			self rangeType: #unfinishedString start: first end: source size.
			self error	": 'unfinished string'"].
	c ~= $' or: [
		self peekChar = $' 
			ifTrue: [sourcePosition := sourcePosition + 1.true] 
			ifFalse: [false]]
	] whileTrue: [sourcePosition := sourcePosition + 1].
	last := sourcePosition.
	self nextChar.
	self scanPast: #stringSymbol start: first - 1 end: last! !

!SHParserST80 methodsFor: 'error handling' stamp: 'tween 4/17/2004 23:54'!
failUnless: aZeroArgumentBlock
	aZeroArgumentBlock value
		ifFalse:[self error]! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 4/13/2004 20:20'!
classOrMetaClass: aClass
    classOrMetaClass := aClass! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/18/2004 12:22'!
parseBinaryMessagePattern   

   	self scanPast:  #patternBinary. 
	self failUnless:[self isName].
	self scanPast: #patternArg.

! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:38'!
isIncompleteMethodArgName: aString 
	"Answer true if aString is the start of the name of a method argument, false otherwise.
    Does not check whether aString is also a blockArgName"

	^(arguments at: 0 ifAbsent: [#()]) anySatisfy: [:each | each beginsWith: aString]! !

!SHParserST80 methodsFor: 'parse support' stamp: 'tween 4/28/2004 10:05'!
pushTemporary: aString 
	(temporaries at: blockDepth ifAbsentPut: [OrderedCollection new: 10]) 
		add: aString! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 09:44'!
isName
	^currentTokenFirst isLetter and: [currentToken last isAlphaNumeric]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:28'!
parseCharSymbol
	| s e |
	s := sourcePosition - 1.
	e := sourcePosition.
	self nextChar.
	self scanPast: #symbol start: s end: e! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:17'!
scanPast: rangeType level: level
	"first level adds no suffix to the rangeType.
	Suffix from 1 to 7 added in cycles , ((level-2) mod(7) + 1)"
	| cycle typePlusCycle |
	
	cycle := level <= 1 
		ifTrue: [0]
		ifFalse:[ ((level - 2) \\ 7) + 1].
	typePlusCycle := cycle = 0 
		ifTrue:[rangeType]
		ifFalse:[(rangeType, cycle asString) asSymbol].
	^self scanPast: typePlusCycle
! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:37'!
isIncompleteBlockTempName: aString 
	"Answer true if aString is the start of the name of a block temporary. false otherwise"

	| temp  |
	blockDepth to: 1 by: -1 do: [:level | 
		temp := (temporaries at: level ifAbsent: [#()]) anySatisfy: [:each | each beginsWith: aString].
		temp ifTrue: [^true]].
	^false! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 5/7/2004 09:31'!
parseBinary 
	| binary type |
	self parseUnary.
	[self isBinary] 
		whileTrue: [
			binary := currentToken.
			type := #binary.
			(binary isEmpty or:[Symbol hasInterned: binary ifTrue: [:sym | ]])
				ifFalse:[
					type := (Symbol thatStartsCaseSensitive: binary skipping: nil) isNil
						ifTrue: [#undefinedBinary]
						ifFalse:[#incompleteBinary]].	
			self scanPast: type. 	
			self parseTerm.
            	self parseUnary]
! !

!SHParserST80 methodsFor: 'accessing' stamp: 'tween 5/9/2004 12:44'!
initialize
	environment := Smalltalk! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 5/7/2004 09:26'!
parseKeyword 
    | keyword rangeIndices type |
    self parseBinary.
	keyword := ''.
	rangeIndices := #().
	[
    		[self isKeyword]
        		whileTrue: [
				keyword := keyword, currentToken. 
				self rangeType: #keyword.
				"remember where this keyword token is in ranges"
				rangeIndices := rangeIndices copyWith: ranges size.
				self scanNext.
				self parseTerm.
				self parseBinary ]
	] ensure: [
		"do this in an ensure so that it happens even if the errorBlock evaluates before getting here"
		"patch up the keyword tokens, so that incomplete and undefined ones look different"
		(keyword isEmpty or:[Symbol hasInterned: keyword ifTrue: [:sym | ]])
			ifFalse:[
				type := (Symbol thatStartsCaseSensitive: keyword skipping: nil) isNil
					ifTrue: [#undefinedKeyword]
					ifFalse:[#incompleteKeyword].
				rangeIndices do: [:i | (ranges at: i) type: type]]]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:05'!
parseSymbol
	| c |
	currentToken = '#' 
		ifTrue: [
			"if token is just the #, then scan whitespace and comments
			and then process the next character.
			Squeak allows space between the # and the start of the symbol 
			e.g. # (),  #  a, #  'sym' "
			self rangeType: #symbol.
			self scanWhitespace].
	c := self currentChar.
	self failWhen: [c isNil or: [c isSeparator]].
	c = $( 
		ifTrue: [
			self nextChar.
			self scanPast: #arrayStart start: currentTokenSourcePosition end: currentTokenSourcePosition + 1.
			^self parseArray].
	c = $' ifTrue: [^self parseSymbolString].
	((self isSelectorCharacter: c) or: [c = $-]) 
		ifTrue: [^self parseSymbolSelector].
	(c isLetter or: [c = $:]) ifTrue: [^self parseSymbolIdentifier].
	^self parseCharSymbol! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 14:18'!
scanWhitespace
	| c |
	
	[c := self currentChar.
	c notNil and: [c isSeparator]] 
		whileTrue: [sourcePosition := sourcePosition + 1].
	c = $" ifTrue: [self scanComment]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:26'!
parseBlockArguments
	[currentTokenFirst = $:] 
		whileTrue: [
			self scanPast: #blockArgColon.
			self failUnless: [self isName].
			self scanPast: #blockPatternArg].
	currentTokenFirst = $| 
		ifTrue: [^self scanPast: #blockArgsBar]! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:24'!
isBlockTempName: aString 
	"Answer true if aString is the name of a block temporary. false otherwise"

	| temp arg |
	blockDepth to: 1 by: -1 do: [:level | 
		arg := (arguments at: level ifAbsent: [#()]) includes: aString.
		arg ifTrue: [^false].
		temp := (temporaries at: level ifAbsent: [#()]) includes: aString.
		temp ifTrue: [^true]].
	^false! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/9/2004 12:46'!
resolvePartial: aString 
	"check if any identifier begins with aString"
	
	(#('self' 'super' 'true' 'false' 'nil' 'thisContext') anySatisfy: [:each | each beginsWith: aString]) 
		ifTrue: [^#incompleteIdentifier].
	(self isIncompleteBlockTempName: aString) ifTrue: [^#incompleteIdentifier].
	(self isIncompleteBlockArgName: aString) ifTrue: [^#incompleteIdentifier].
	(self isIncompleteMethodTempName: aString) ifTrue: [^#incompleteIdentifier].
	(self isIncompleteMethodArgName: aString) ifTrue: [^#incompleteIdentifier].
	(instanceVariables anySatisfy: [:each | each beginsWith: aString]) ifTrue: [^#incompleteIdentifier].
	workspace 
		ifNotNil: [(workspace hasBindingThatBeginsWith: aString) ifTrue: [^#incompleteIdentifier]].
	classOrMetaClass notNil 
		ifTrue: [
			classOrMetaClass theNonMetaClass withAllSuperclasses do: [:c | 
				(c classPool hasBindingThatBeginsWith: aString) ifTrue: [^#incompleteIdentifier].
				c sharedPools do: [:p | (p hasBindingThatBeginsWith: aString) ifTrue: [^#incompleteIdentifier]].
				(c environment hasBindingThatBeginsWith: aString) ifTrue: [^#incompleteIdentifier]]]
		ifFalse: [(environment hasBindingThatBeginsWith: aString) ifTrue: [^#incompleteIdentifier]].
	^#undefinedIdentifier! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:05'!
parseString
	| first c answer last |
	first := sourcePosition.
	answer := ''.
	
	[(c := self currentChar) isNil 
		ifTrue: [
			self rangeType: #unfinishedString start: first - 1 end: source size.
			self error	": 'unfinished string'"].
	(c ~= $' 	
		ifTrue: [answer := answer copyWith: c. true] 
		ifFalse: [false]
	) or: [
		self peekChar = $' 
			ifTrue: [
				sourcePosition := sourcePosition + 1.
				answer := answer copyWith: $'.
				true]
			ifFalse: [false]]
	] whileTrue: [sourcePosition := sourcePosition + 1].
	last := sourcePosition.
	self nextChar.
	self scanPast: #string start: first - 1 end: last.
	^answer! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 14:20'!
isBinary
	(currentToken isNil or: [self isName or: [self isKeyword]]) 
		ifTrue: [^false].
	1 to: currentToken size do: [:i | | c |
		c := currentToken at: i.
		((self isSelectorCharacter: c) or: [i = 1 and: [c = $-]]) 
			ifFalse: [^false]].
	^true! !

!SHParserST80 methodsFor: 'parse support' stamp: 'tween 4/28/2004 10:03'!
pushArgument: aString 
	(arguments at: blockDepth ifAbsentPut: [OrderedCollection new: 10]) 
		add: aString! !

!SHParserST80 methodsFor: 'parse support' stamp: 'tween 4/28/2004 09:48'!
enterBlock
	blockDepth := blockDepth + 1.
	bracketDepth := bracketDepth + 1! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 09:44'!
parseStatementList
	
	[[currentTokenFirst = $.] whileTrue: [self scanPast: #statementSeparator].
	(currentToken notNil and: [currentTokenFirst ~= $]]) 
		ifTrue: [self parseStatement].
	currentTokenFirst = $.] 
			whileTrue: [self scanPast: #statementSeparator]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:29'!
parseMethodTemporaries
	currentTokenFirst = $| 
		ifTrue: [
			self scanPast: #methodTempBar.
			[self isName] 
				whileTrue: [self scanPast: #patternTempVar].
			self failUnless: [currentToken = '|'].
			self scanPast: #methodTempBar]! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/7/2004 06:39'!
isIncompleteMethodTempName: aString 
	"Answer true if aString is the start of then name of a method temporary, false otherwise."

	^(temporaries at: 0 ifAbsent: [#()]) anySatisfy: [:each | each beginsWith: aString]! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 10:06'!
scanPast: rangeType 
	"record rangeType for current token .
	record argument and temp declarations.
	scan and answer the next token"
	rangeType = #blockPatternArg ifTrue: [self pushArgument: currentToken].
	rangeType = #blockPatternTempVar ifTrue: [self pushTemporary: currentToken].
	rangeType = #patternArg ifTrue: [self pushArgument: currentToken].
	rangeType = #patternTempVar ifTrue: [self pushTemporary: currentToken].
	^self
		rangeType: rangeType;
		scanNext! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:29'!
parseStatementListForBraceArray
	"same as parseStatementList, but does not allow empty statements e.g {...$a...}.
	A single terminating . IS allowed e.g. {$a.} "

	
	[currentTokenFirst ~= $} ifTrue: [self parseStatement].
	currentTokenFirst = $.] 
		whileTrue: [self scanPast: #statementSeparator]! !

!SHParserST80 methodsFor: 'error handling' stamp: 'tween 4/17/2004 23:53'!
failWhen: aZeroArgumentBlock
	aZeroArgumentBlock value
		ifTrue:[self error]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 09:44'!
parseBraceArray
	self parseStatementListForBraceArray.
	self failUnless: [currentTokenFirst = $}].
	self scanPast: #rightBrace! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 09:46'!
scanIdentifier
	| c start |
	start := sourcePosition.
	[(c := self nextChar) isAlphaNumeric] whileTrue: [].
	(c = $: and: [(self isSelectorCharacter: self peekChar) not]) 
		ifTrue: [self nextChar].
	currentToken := source copyFrom: start to: sourcePosition - 1.
	currentTokenSourcePosition := start! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:29'!
parseLiteral: inArray 
	currentTokenFirst = $$ 
		ifTrue: [
			| pos |
			self failWhen: [self currentChar isNil].
			self rangeType: #'$'.
			pos := currentTokenSourcePosition + 1.
			self nextChar.
			^self scanPast: #character start: pos end: pos].
	currentTokenFirst isDigit 
		ifTrue: [
			"do not parse the number, can be time consuming"
			^self scanPast: #number].
	currentToken = '-' 
		ifTrue: [
			| c |
			c := self currentChar.
			(inArray and: [c isNil or: [c isDigit not]]) 
				ifTrue: [
					"single - can be a symbol in an Array"
					^self scanPast: #symbol].
			self scanPast: #-.
			self failWhen: [currentToken isNil].
			"token isNil ifTrue: [self error: 'Unexpected End Of Input']."
			"do not parse the number, can be time consuming"
			^self scanPast: #number].
	currentTokenFirst = $' ifTrue: [^self parseString].
	currentTokenFirst = $# ifTrue: [^self parseSymbol].
	(inArray and: [currentToken notNil]) ifTrue: [^self scanPast: #symbol].
	self failWhen: [currentTokenFirst = $.].
	self error	": 'argument missing'"! !

!SHParserST80 methodsFor: 'identifier testing' stamp: 'tween 5/9/2004 12:46'!
resolve: aString 
	(#('self' 'super' 'true' 'false' 'nil' 'thisContext') includes: aString) 
		ifTrue: [^aString asSymbol].
	(self isBlockTempName: aString) ifTrue: [^#blockTempVar].
	(self isBlockArgName: aString) ifTrue: [^#blockArg].
	(self isMethodTempName: aString) ifTrue: [^#tempVar].
	(self isMethodArgName: aString) ifTrue: [^#methodArg].
	(instanceVariables includes: aString) ifTrue: [^#instVar].
	workspace 
		ifNotNil: [(workspace hasBindingOf: aString) ifTrue: [^#workspaceVar]].
	Symbol hasInterned: aString ifTrue: [:sym | 
		classOrMetaClass notNil 
			ifTrue: [
				classOrMetaClass theNonMetaClass withAllSuperclasses do: [:c | 
					(c classPool bindingOf: sym) ifNotNil: [^#classVar].
					c sharedPools do: [:p | (p bindingOf: sym) ifNotNil: [^#poolConstant]].
					(c environment bindingOf: sym) ifNotNil: [^#globalVar]]]
			ifFalse: [(environment bindingOf: sym) ifNotNil: [^#globalVar]]].
	^self resolvePartial: aString! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:11'!
parseTerm
	self failWhen: [currentToken isNil].
	currentTokenFirst = $( 
		ifTrue: [
			bracketDepth := bracketDepth + 1.
			self scanPast: #leftParenthesis level: bracketDepth.
			self parseExpression.
			self failUnless: [currentTokenFirst = $)].
			self scanPast: #rightParenthesis level: bracketDepth.
			^bracketDepth := bracketDepth - 1].
	currentTokenFirst = $[ ifTrue: [^self parseBlock].
	currentTokenFirst = ${ 
		ifTrue: [
			self scanPast: #leftBrace.
			^self parseBraceArray].
	self isName ifTrue: [^self scanPast: (self resolve: currentToken)].
	self parseLiteral: false! !

!SHParserST80 methodsFor: 'token testing' stamp: 'tween 4/28/2004 09:43'!
isAnsiAssignment
	^currentToken = ':='! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 09:42'!
nextChar
	sourcePosition := sourcePosition + 1.
	^source at: sourcePosition ifAbsent: [$ ]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 09:44'!
parseArray
	[currentTokenFirst = $)] whileFalse: [self parseLiteralArrayElement].
	self scanPast: #arrayEnd! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 14:27'!
parseBlockTemporaries
	currentTokenFirst = $| 
		ifTrue: [
			self scanPast: #blockTempBar.
			[self isName] 
				whileTrue: [self scanPast: #blockPatternTempVar].
			self failUnless: [currentToken = '|'].
			self scanPast: #blockTempBar]! !

!SHParserST80 methodsFor: 'parse' stamp: 'tween 4/28/2004 10:16'!
parse: isAMethod 
	"Parse the receiver's text. If isAMethod is true
    then treat text as a method, if false as an
    expression with no message pattern"

	self initializeInstanceVariables.
	sourcePosition := 1.
	arguments := Dictionary new.
	temporaries := Dictionary new.
	blockDepth := bracketDepth := 0.
	ranges isNil 
		ifTrue: [ranges := OrderedCollection new: 100]
		ifFalse: [ranges reset].
	errorBlock := [^false].
	self scanNext.
	isAMethod ifTrue: [self parseMessagePattern].
	self parseMethodTemporaries.
	self parsePrimitiveOrExternalCall.
	self parseStatementList.
	currentToken ifNotNil: [self error].
	^true! !

!SHParserST80 methodsFor: 'scan' stamp: 'tween 4/28/2004 09:42'!
currentChar
	^source at: sourcePosition ifAbsent: [nil]! !

!SHParserST80 methodsFor: 'parse support' stamp: 'tween 4/28/2004 09:48'!
leaveBlock
	arguments removeKey: blockDepth ifAbsent: [].
	temporaries removeKey: blockDepth ifAbsent: [].
	blockDepth := blockDepth - 1.
	bracketDepth := bracketDepth - 1! !

Object subclass: #SHRange
	instanceVariableNames: 'start end type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Parsing'!

!SHRange commentStamp: 'tween 8/16/2004 15:16' prior: 0!
I associate a type with a range of characters in a String
I have these instance variables...
	start - the one based index of the first character of the range within the String.
	end - the one based index of the last character  of the range within the String.
	type - a Symbol describing the type of the range
	
A sequence of instances of me are created by an instance of SHParserST80 which can then used by an instance of  SHTextStyler to style Text. !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:55'!
type: aSymbol
	type := aSymbol! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:55'!
end
	^end! !

!SHRange class methodsFor: 'instance creation' stamp: 'tween 4/28/2004 10:19'!
start: s end: e type: aSymbol
	
	^self new
		start: s;
		end: e;
		type: aSymbol;
		yourself! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:55'!
type
	^type! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:54'!
start
	^start! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:54'!
start: anInteger
	start := anInteger! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/13/2004 18:55'!
end: anInteger
	end := anInteger! !

!SHRange methodsFor: 'accessing' stamp: 'tween 4/24/2004 15:48'!
length
	^end - start + 1! !

TextMorphForEditView subclass: #TextMorphForShout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Text Support'!

!TextMorphForShout methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:39'!
paragraph
	"Paragraph instantiation is lazy -- create it only when needed"
	paragraph ifNotNil: [^ paragraph].
	self setProperty: #CreatingParagraph toValue: true.
	self setDefaultContentsIfNil.
	"...Code here to recreate the paragraph..."
	paragraph := (self paragraphClass new textOwner: self owner).
	paragraph wantsColumnBreaks: successor notNil.
	paragraph
		compose: text
		style: textStyle copy
		from: self startingIndex
		in: self container.
	wrapFlag ifFalse:
		["Was given huge container at first... now adjust"
		paragraph adjustRightX].
	paragraph focused: (ActiveHand notNil and:[ActiveHand keyboardFocus == self]).
	self fit.
	self removeProperty: #CreatingParagraph.
	^ paragraph! !

!TextMorphForShout methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:25'!
installEditorToReplace: priorEditor
	"Install an editor for my paragraph.  This constitutes 'hasFocus'.
	If priorEditor is not nil, then initialize the new editor from its state.
	We may want to rework this so it actually uses the prior editor."

	| stateArray |
	priorEditor ifNotNil: [stateArray := priorEditor stateArray].
	editor := TextMorphForShoutEditor new morph: self.
	editor changeParagraph: self paragraph.
	priorEditor ifNotNil: [editor stateArrayPut: stateArray].
	self selectionChanged.
	^ editor! !

!Symbol class methodsFor: '*Shout-Parsing' stamp: 'tween 5/7/2004 09:22'!
thatStartsCaseSensitive: leadingCharacters skipping: skipSym
	"Same as thatStarts:skipping: but caseSensitive"
	| size firstMatch key |

	size := leadingCharacters size.
	size = 0 ifTrue: [^skipSym ifNil: [#''] ifNotNil: [nil]].
	firstMatch := leadingCharacters at: 1.
	size > 1 ifTrue: [key := leadingCharacters copyFrom: 2 to: size].
	self allSymbolTablesDo: [:each |
			each size >= size ifTrue:
				[
					((each at: 1) == firstMatch and:
						[key == nil or:
							[(each findString: key startingAt: 2 caseSensitive: true) = 2]])
								ifTrue: [^each]
				]
		] after: skipSym.

	^nil
! !

!SharedPool class methodsFor: '*Shout-Parsing' stamp: 'tween 5/7/2004 07:08'!
hasBindingThatBeginsWith: aString
	"Answer true if the receiver has a binding that begins with aString, false otherwise"

	"First look in classVar dictionary."
	(self classPool hasBindingThatBeginsWith: aString) ifTrue:[^true].
	"Next look in shared pools."
	self sharedPools do:[:pool | 
		(pool hasBindingThatBeginsWith: aString) ifTrue: [^true]].
	^false! !

TextMorphEditor subclass: #TextMorphForShoutEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Text Support'!

!TextMorphForShoutEditor methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:28'!
clearParens
	lastParenLocation ifNotNil:
		[self text string size >= lastParenLocation ifTrue: [
			self text
				removeAttribute: TextEmphasis bold
				from: lastParenLocation
				to: lastParenLocation].
		lastParenLocation := nil]
! !

!TextMorphForShoutEditor methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 17:20'!
correctFrom: start to: stop with: aString
	
	view correctFrom: start to: stop with: aString.
	^super correctFrom: start to: stop with: aString
! !

!TextMorphForShoutEditor methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:28'!
blinkParen
	lastParenLocation ifNotNil:
		[self text string size >= lastParenLocation ifTrue: [
			self text
				addAttribute: TextEmphasis bold
				from: lastParenLocation
				to: lastParenLocation]]
! !

!Text methodsFor: '*Shout-accessing' stamp: 'tween 4/1/2004 18:15'!
runs: anArray

	runs _ anArray! !

PluggableTextMorph subclass: #PluggableShoutMorph
	instanceVariableNames: 'styler unstyledAcceptText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Windows'!

!PluggableShoutMorph commentStamp: '<historical>' prior: 0!
I am a subclass of PluggableTextMorph.

Instances of me are usually created using my #on:text:accept:readSelection:menu: class method.

In order to colour the text, I use an instance of SHTextStylerST80, which I store in my 'styler' instance variable.

When my setText: method is called, I use my styler to ...
	a) optionally set all assignments to ansi or leftArrow. 
	b) Colour my text (immediately, if the text is less than 4096 chars in length, or in a backgroundProcess otherwise)

	When my text is changed, my hasUnacceptedEdits: method is called with true, and I ask my styler to re-colour my text. This is performed in a background process so that typing remains responsive regardless of the length of the text.

	Just before my styler is about to format/style the text, I send #stylerAboutToStyle:  to my model. This gives my model a chance to veto the styling (by answering false), or to initialize the styler with information it needs in order to parse the text correctly (e.g. the class to which a method belongs, or the workspace in which I am contained).

	My styler informs me that it has finished styling by triggering the #stylerStyled: and #stylerStyledInBackground: events which I handle. I then update the textAttributes of my text and refresh the display.
 
	My 'unstyledAcceptText' instance variable is used in conjunction with my #acceptTextInModel and #correctFrom:to:with: methods to ensure that when my text is modified during a method compilation (removing unused vars etc), I do not lose those changes.
	!

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 4/27/2004 22:01'!
styler: anObject
	"Set the receiver's styler to <anObject>.
	<anObject> will be asked to restyle the textMorph's text when it is initially set,
	and after each subsequent change"
	
	styler := anObject.
	styler ifNotNil: [
		styler 
			when: #styled: send: #stylerStyled: to: self;
			when: #styledInBackground: send: #stylerStyledInBackground: to: self ]! !

!PluggableShoutMorph class methodsFor: 'class initialization' stamp: 'tween 5/8/2004 16:21'!
initialize
	"Register the receiver with MorphiTextEditor in the AppRegistry.
	If the old default was a plain old PluggableTextMorph, then make the receiver the new default, otherwise make the default nil so that the user is prompted.
	This is run after the monticello package is loaded"
	| current |
	
	current := MorphicTextEditor defaultOrNil.
	(current isNil or: [current = PluggableTextMorph or: [current = self]])
		ifTrue:[MorphicTextEditor default: self]
		ifFalse:[	MorphicTextEditor register: self; default: nil]! !

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 8/24/2004 14:25'!
setText: aText
	| formattedText copyText |
	
	formattedText := aText.
	styler notNil
		ifTrue: [ formattedText := styler format: formattedText asText].
	scrollBar setValue: 0.0.
	textMorph
		ifNil: [textMorph := TextMorphForShout new
						contents: formattedText wrappedTo: self innerBounds width-6.
				textMorph setEditView: self.
				scroller addMorph: textMorph]
		ifNotNil: [textMorph newContents: formattedText].
	self hasUnacceptedEdits: false.
	self setScrollDeltas.
	styler notNil
		ifTrue: [
			copyText :=  textMorph contents copy.
			"style the text immediately if less than 4K, otherwise style in a background process"
			copyText size < 4096
				ifTrue:[	styler style: copyText]
				ifFalse:[styler styleInBackgroundProcess:  copyText]]! !

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 6/28/2004 08:11'!
correctFrom: start to: stop with: aString
	"see the comment in #acceptTextInModel "
	unstyledAcceptText ifNotNil:[unstyledAcceptText replaceFrom: start to: stop with: aString ].
	^ super correctFrom: start to: stop with: aString! !

!PluggableShoutMorph methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:45'!
accept 
	"Inform the model of text to be accepted, and return true if OK."

	| ok saveSelection saveScrollerOffset |
"sps 8/13/2001 22:41: save selection and scroll info"
	saveSelection := self selectionInterval copy.
	saveScrollerOffset := scroller offset copy.

	(self canDiscardEdits and: [(self hasProperty: #alwaysAccept) not])
		ifTrue: [^ self flash].

	self hasEditingConflicts ifTrue:
		[(self confirm: 
'Caution!! This method may have been
changed elsewhere since you started
editing it here.  Accept anyway?' translated) ifFalse: [^ self flash]].
	ok := self acceptTextInModel.
	ok==true ifTrue:
		[self setText: self getText.
		self hasUnacceptedEdits: false.
		(model dependents detect: [:dep | (dep isKindOf: PluggableTextMorph) and: [dep getTextSelector == #annotation]] ifNone: [nil]) ifNotNilDo:
			[:aPane | model changed: #annotation]].

	"sps 8/13/2001 22:41: restore selection and scroll info"
	["During the step for the browser, updateCodePaneIfNeeded is called, and 
		invariably resets the contents of the codeholding PluggableTextMorph
		at that time, resetting the cursor position and scroller in the process.
		The following line forces that update without waiting for the step, 		then restores the cursor and scrollbar"

	ok ifTrue: "(don't bother if there was an error during compile)"
		[(model isKindOf: CodeHolder) 
			ifTrue: [model updateCodePaneIfNeeded].
		WorldState addDeferredUIMessage:
			[self currentHand newKeyboardFocus: textMorph.
			scroller offset: saveScrollerOffset.
			self setScrollDeltas.
			self selectFrom: saveSelection first to: saveSelection last]]]

			on: Error do: []
! !

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 4/27/2004 22:02'!
stylerStyledInBackground: styledCopyOfText 
	"It is possible that the text string	has changed since the styling began. Disregard the styles if styledCopyOfText's string differs with the current textMorph contents string"
	
	textMorph contents string = styledCopyOfText string
		ifTrue: [self stylerStyled: styledCopyOfText]
	! !

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 4/27/2004 22:01'!
hasUnacceptedEdits: aBoolean
	"Set the hasUnacceptedEdits flag to the given value. "
	aBoolean == hasUnacceptedEdits ifFalse:
		[hasUnacceptedEdits := aBoolean.
		self changed].
	aBoolean ifFalse: [hasEditingConflicts := false].
	(aBoolean and: [styler notNil])
		ifTrue: [ styler styleInBackgroundProcess: textMorph contents copy]! !

!PluggableShoutMorph methodsFor: 'styler' stamp: 'tween 4/27/2004 22:01'!
stylerStyled: styledCopyOfText

	"TO DO: move this functionality into TextMorph"
	textMorph contents runs: styledCopyOfText runs .
	textMorph paragraph recomposeFrom: 1 to: textMorph contents size delta: 0.
	textMorph updateFromParagraph.
	selectionInterval 
		ifNotNil:[
			textMorph editor
				selectInvisiblyFrom: selectionInterval first to: selectionInterval last;
				storeSelectionInParagraph;
				setEmphasisHere].
	textMorph editor blinkParen.
	self scrollSelectionIntoView! !

!PluggableShoutMorph class methodsFor: 'instance creation' stamp: 'tween 8/24/2004 15:52'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel
	|styler |
	styler _ SHTextStylerST80 new.
	styler when: #aboutToStyle send: #shoutStylerAboutToStyle: to: anObject with: styler.
	^ self new
		styler: styler;
		on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: getSelectionSel
		menu: getMenuSel! !

!PluggableShoutMorph methodsFor: 'override' stamp: 'tween 8/24/2004 14:48'!
acceptTextInModel 
	| answer textToAccept |
	
	(styler isNil or:[styler shouldRemoveStylesFromAcceptText not])
		ifTrue:[
			unstyledAcceptText := nil.
			textToAccept := textMorph asText.
			^setTextSelector isNil or:
				[setTextSelector numArgs = 2
					ifTrue: [model perform: setTextSelector with: textToAccept with: self]
					ifFalse: [model perform: setTextSelector with: textToAccept]]].			
	"#correctFrom:to:with: is sent when the method source is
	manipulated during compilation (removing unused temps,
	changing selectors etc). But 	#correctFrom:to:with: operates 
	on the textMorph's text, and we may be saving an unstyled 
	copy of the text. This means that these corrections will be lost
	unless we also apply the corrections to our unstyled copy.
		So remember our unstyled text in unstyledAcceptText, so
	that when #correctFrom:to:with: is received we can also apply
	the correction to it"
	unstyledAcceptText := styler unstyledTextFrom: textMorph asText.
	answer :=setTextSelector isNil or:
		[setTextSelector numArgs = 2
			ifTrue: [model perform: setTextSelector with: unstyledAcceptText with: self]
			ifFalse: [model perform: setTextSelector with: unstyledAcceptText]].
	unstyledAcceptText := nil.
	^answer! !

!MessageSet methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:52'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"
	
	Preferences syntaxHighlightingAsYouType ifFalse: [^false].
	self showingSource ifFalse: [^false].
	aTextStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !

!Dictionary methodsFor: '*Shout-Parsing' stamp: 'tween 5/7/2004 07:10'!
hasBindingThatBeginsWith: aString
	"Answer true if the receiver has a key that begins with aString, false otherwise"
	
	self keysDo:[:each | 
		(each beginsWith: aString) 
			ifTrue:[^true]].
	^false! !

PluggableTextView subclass: #PluggableShoutView
	instanceVariableNames: 'styler unstyledAcceptText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shout-Pluggable Views'!

!PluggableShoutView commentStamp: 'tween 8/16/2004 15:09' prior: 0!
I am a subclass of PluggableTextView.

Instances of me are usually created using my #on:text:accept:readSelection:menu: class method.

In order to colour the text, I use an instance of SHTextStylerST80, which I store in my 'styler' instance variable.

When my #editString: method is called, I use the styler to ...
	a) optionally set all assignments to ansi or leftArrow in my text. 
	b) Colour my text (immediately, if the text is less than 4096 chars in length, or in a backgroundProcess otherwise)

	When my text is changed, my #hasUnacceptedEdits: method is called with true, and I ask my styler to re-colour my text. This is performed in a background process so that typing remains responsive regardless of the length of the text.

	Just before my styler is about to format/style the text, I send #stylerAboutToStyle:  to my model. This gives my model a chance to veto the styling (by answering false), or to initialize the styler with information it needs in order to parse the text correctly (e.g. the class to which a method belongs, or the workspace in which I am contained).

	My styler informs me that it has finished styling by triggering the #stylerStyled: and #stylerStyledInBackground: events which I handle. I then update the textAttributes of my text and refresh the display.
 
	My 'unstyledAcceptText' instance variable is used in conjunction with my #correctFrom:to:with: method to ensure that when my text is modified during a method compilation (removing unused vars etc), I do not lose those changes.
	!

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 7/27/2004 16:31'!
stylerStyled: styledCopyOfText

	controller deselectDuring: [
		displayContents text runs: styledCopyOfText runs.
		displayContents composeAll.
		controller recomputeInterval.
		controller setEmphasisHere.
		controller blinkParen.
		viewport ifNotNil:[
			unstyledAcceptText ifNil:[
				"don't displayView if accepting text (i.e. unstyledAcceptText not nil)
				because popup menus are painted over if we do" 
				self displayView]]]! !

!PluggableShoutView class methodsFor: 'instance creation' stamp: 'tween 8/24/2004 15:53'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel
	|styler |
	styler _ SHTextStylerST80 new.
	styler when: #aboutToStyle send: #shoutStylerAboutToStyle: to: anObject with: styler.
	^ self new 
		styler: styler;
		on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: getSelectionSel
		menu: getMenuSel! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 7/27/2004 11:15'!
setText: textToAccept from: ctlr
	"Inform the model of text to be accepted, and return true if OK.
	Any errors should be reported to the controller, ctlr."
	|answer|
	setTextSelector == nil ifTrue: [^ true].
	unstyledAcceptText := (styler notNil and: [styler shouldRemoveStylesFromAcceptText])
		ifTrue: [styler unstyledTextFrom: textToAccept]
		ifFalse: [textToAccept].
	answer := setTextSelector numArgs = 2
		ifTrue: [model perform: setTextSelector with: unstyledAcceptText with: ctlr]
		ifFalse: [model perform: setTextSelector with: unstyledAcceptText].
	unstyledAcceptText := nil.	
	^answer! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 4/27/2004 21:54'!
styler: anObject
	"Set the receiver's styler to <anObject>.
	<anObject> will be asked to restyle the displayContent's text when it is initially set,
	and after each subsequent change"
	
	styler := anObject.
	styler ifNotNil: [
		styler 
			when: #styled: send: #stylerStyled: to: self;
			when: #styledInBackground: send: #stylerStyledInBackground: to: self ]! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 8/24/2004 17:16'!
hasUnacceptedEdits: aBoolean
	super hasUnacceptedEdits: aBoolean.
	(aBoolean and: [styler notNil])
		ifTrue: [ styler styleInBackgroundProcess: displayContents text copy ]! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 4/27/2004 21:53'!
editString: aString 
	| formattedText |
	
	formattedText := aString.
	styler notNil
		ifTrue: [ formattedText := styler format: formattedText asText]. 
	super editString: formattedText.
	styler notNil
		ifTrue: [ styler style: displayContents text copy] ! !

!PluggableShoutView methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 14:34'!
defaultControllerClass 

	^ PluggableShoutViewController! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 4/27/2004 21:54'!
stylerStyledInBackground: styledCopyOfText

	(displayContents text string = styledCopyOfText string)
		ifTrue: [self stylerStyled: styledCopyOfText]! !

!PluggableShoutView class methodsFor: 'as yet unclassified' stamp: 'tween 8/24/2004 15:25'!
initialize
	"Register the receiver with MvcTextEditor in the AppRegistry.
	If the old default was a plain old PluggableTextView, then make the receiver the new default, otherwise make the default nil so that the user is prompted.
	This is run after the monticello package is loaded"
	| current |
	
	current := MvcTextEditor defaultOrNil.
	(current isNil or: [current = PluggableTextView or: [current = self]])
		ifTrue:[MvcTextEditor default: self]
		ifFalse:[	MvcTextEditor register: self; default: nil]! !

!PluggableShoutView methodsFor: 'styler' stamp: 'tween 7/27/2004 11:42'!
correctFrom: start to: stop with: aString

	unstyledAcceptText ifNotNil:[unstyledAcceptText replaceFrom: start to: stop with: aString ].! !

!CodeHolder methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:51'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"
	
	^false! !

!FileContentsBrowser methodsFor: '*Shout-Styling' stamp: 'tween 8/24/2004 15:51'!
shoutStylerAboutToStyle: aTextStyler
	"set up the compilation class in aTextStyler.
	Answer true if styling should go ahead, false otherwise"

	Preferences syntaxHighlightingAsYouType ifFalse: [^false].
	self showingSource ifFalse: [^false].
	aTextStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !
SHTextStylerST80 initialize!
PluggableShoutMorph initialize!
PluggableShoutView initialize!
